@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: netrc.h,v 1.11 2004/01/07 09:19:35 bagder Exp $
 /* ######################################################################
 
    netrc file parser - returns the login and password of a give host in
@@ -14,12 +13,9 @@
 #ifndef NETRC_H
 #define NETRC_H
 
-#include <memory>
 #include <string>
-#include <vector>
 
 #include <apt-pkg/macros.h>
-#include <apt-pkg/pkgcache.h>
 
 #ifndef APT_8_CLEANER_HEADERS
 #include <apt-pkg/strutl.h>
@@ -36,5 +32,4 @@ class FileFd;
 APT_DEPRECATED_MSG("Use FileFd-based MaybeAddAuth instead")
 void maybe_add_auth(URI &Uri, std::string NetRCFile);
 bool MaybeAddAuth(FileFd &NetRCFile, URI &Uri);
-bool IsAuthorized(pkgCache::PkgFileIterator const I, std::vector<std::unique_ptr<FileFd>> &authconfs) APT_HIDDEN;
 #endif
