@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: deblistparser.h,v 1.9 2001/02/20 07:03:17 jgg Exp $
 /* ######################################################################
    
    Debian Package List Parser - This implements the abstract parser 
@@ -11,11 +10,11 @@
 #ifndef PKGLIB_DEBLISTPARSER_H
 #define PKGLIB_DEBLISTPARSER_H
 
-#include <apt-pkg/pkgcachegen.h>
-#include <apt-pkg/tagfile.h>
+#include <apt-pkg/macros.h>
 #include <apt-pkg/md5.h>
 #include <apt-pkg/pkgcache.h>
-#include <apt-pkg/macros.h>
+#include <apt-pkg/pkgcachegen.h>
+#include <apt-pkg/tagfile.h>
 
 #include <string>
 #include <vector>
@@ -87,9 +86,6 @@ class APT_HIDDEN debListParser : public pkgCacheListParser
 
    virtual bool Step() APT_OVERRIDE;
 
-   bool LoadReleaseInfo(pkgCache::RlsFileIterator &FileI,FileFd &File,
-			std::string const &section);
-
    APT_PUBLIC static const char *ParseDepends(const char *Start,const char *Stop,
 	 std::string &Package,std::string &Ver,unsigned int &Op);
    APT_PUBLIC static const char *ParseDepends(const char *Start,const char *Stop,
