@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: init.cc,v 1.20 2003/02/09 20:31:05 doogie Exp $
 /* ######################################################################
 
    Init - Initialize the package library
@@ -123,7 +122,7 @@ static bool pkgInitArchTupleMap()
 // ---------------------------------------------------------------------
 /* Directories are specified in such a way that the FindDir function will
    understand them. That is, if they don't start with a / then their parent
-   is prepended, this allows a fair degree of flexability. */
+   is prepended, this allows a fair degree of flexibility. */
 bool pkgInitConfig(Configuration &Cnf)
 {
    // General APT things
@@ -151,7 +150,6 @@ bool pkgInitConfig(Configuration &Cnf)
    Cnf.CndSet("Dir::Etc::sourceparts","sources.list.d");
    Cnf.CndSet("Dir::Etc::main","apt.conf");
    Cnf.CndSet("Dir::Etc::netrc", "auth.conf");
-   Cnf.CndSet("Dir::Etc::netrcparts", "auth.conf.d");
    Cnf.CndSet("Dir::Etc::parts","apt.conf.d");
    Cnf.CndSet("Dir::Etc::preferences","preferences");
    Cnf.CndSet("Dir::Etc::preferencesparts","preferences.d");
@@ -251,7 +249,7 @@ bool pkgInitConfig(Configuration &Cnf)
    return good;
 }
 									/*}}}*/
-// pkgInitSystem - Initialize the _system calss				/*{{{*/
+// pkgInitSystem - Initialize the _system class				/*{{{*/
 // ---------------------------------------------------------------------
 /* */
 bool pkgInitSystem(Configuration &Cnf,pkgSystem *&Sys)
