@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: pkgcache.cc,v 1.37 2003/02/10 01:40:58 doogie Exp $
 /* ######################################################################
    
    Package Cache - Accessor code for the cache
@@ -59,7 +58,7 @@ pkgCache::Header::Header()
    /* Whenever the structures change the major version should be bumped,
       whenever the generator changes the minor version should be bumped. */
    APT_HEADER_SET(MajorVersion, 13);
-   APT_HEADER_SET(MinorVersion, 1);
+   APT_HEADER_SET(MinorVersion, 0);
    APT_HEADER_SET(Dirty, false);
 
    APT_HEADER_SET(HeaderSz, sizeof(pkgCache::Header));
@@ -969,13 +968,6 @@ const char * pkgCache::VerIterator::MultiArchType() const
    included here. */
 bool pkgCache::RlsFileIterator::IsOk()
 {
-   struct stat Buf;
-   if (stat(FileName(),&Buf) != 0)
-      return false;
-
-   if (Buf.st_size != (signed)S->Size || Buf.st_mtime != S->mtime)
-      return false;
-
    return true;
 }
 									/*}}}*/
@@ -1003,13 +995,6 @@ string pkgCache::RlsFileIterator::RelStr()
    included here. */
 bool pkgCache::PkgFileIterator::IsOk()
 {
-   struct stat Buf;
-   if (stat(FileName(),&Buf) != 0)
-      return false;
-
-   if (Buf.st_size != (signed)S->Size || Buf.st_mtime != S->mtime)
-      return false;
-
    return true;
 }
 									/*}}}*/
