@@ -20,6 +20,7 @@
 
 #include <apt-private/private-cacheset.h>
 #include <apt-private/private-output.h>
+#include <apt-private/private-install.h>
 #include <apt-private/private-show.h>
 
 #include <ostream>
@@ -30,39 +31,27 @@
 #include <apti18n.h>
 									/*}}}*/
 
-static bool OpenPackagesFile(pkgCacheFile &CacheFile, pkgCache::VerIterator const &V,/*{{{*/
-      FileFd &PkgF, pkgCache::VerFileIterator &Vf)
+pkgRecords::Parser &LookupParser(pkgRecords &Recs, pkgCache::VerIterator const &V, pkgCache::VerFileIterator &Vf) /*{{{*/
 {
-   pkgCache const * const Cache = CacheFile.GetPkgCache();
-   if (unlikely(Cache == NULL))
-      return false;
-
-   // Find an appropriate file
    Vf = V.FileList();
    for (; Vf.end() == false; ++Vf)
       if ((Vf.File()->Flags & pkgCache::Flag::NotSource) == 0)
 	 break;
    if (Vf.end() == true)
       Vf = V.FileList();
-      
-   // Check and load the package list file
-   pkgCache::PkgFileIterator I = Vf.File();
-   if (I.IsOk() == false)
-      return _error->Error(_("Package file %s is out of sync."),I.FileName());
-
-   // Read the record
-   return PkgF.Open(I.FileName(), FileFd::ReadOnly, FileFd::Extension);
+   return Recs.Lookup(Vf);
 }
 									/*}}}*/
-static APT_PURE unsigned char const* skipDescriptionFields(unsigned char const * DescP)/*{{{*/
+static APT_PURE char const *skipDescription(char const *DescP, size_t const Length, bool fields) /*{{{*/
 {
+   auto const backup = DescP;
    char const * const TagName = "\nDescription";
    size_t const TagLen = strlen(TagName);
-   while ((DescP = (unsigned char*)strchr((char*)DescP, '\n')) != NULL)
+   while ((DescP = static_cast<char const *>(memchr(DescP, '\n', Length - (DescP - backup)))) != nullptr)
    {
       if (DescP[1] == ' ')
 	 DescP += 2;
-      else if (strncmp((char*)DescP, TagName, TagLen) == 0)
+      else if (fields && strncmp((char *)DescP, TagName, TagLen) == 0)
 	 DescP += TagLen;
       else
 	 break;
@@ -72,114 +61,173 @@ static APT_PURE unsigned char const* skipDescriptionFields(unsigned char const *
    return DescP;
 }
 									/*}}}*/
-bool DisplayRecordV1(pkgCacheFile &CacheFile, pkgCache::VerIterator const &V,/*{{{*/
-                   std::ostream &out)
+static APT_PURE char const *findDescriptionField(char const *DescP, size_t const Length) /*{{{*/
 {
-   FileFd PkgF;
-   pkgCache::VerFileIterator Vf;
-   if (OpenPackagesFile(CacheFile, V, PkgF, Vf) == false)
-      return false;
-
-   pkgCache * const Cache = CacheFile.GetPkgCache();
-   if (unlikely(Cache == NULL))
-      return false;
-
-   // Read the record (and ensure that it ends with a newline and NUL)
-   unsigned char *Buffer = new unsigned char[Cache->HeaderP->MaxVerFileSize+2];
-   Buffer[Vf->Size] = '\n';
-   Buffer[Vf->Size+1] = '\0';
-   if (PkgF.Seek(Vf->Offset) == false ||
-       PkgF.Read(Buffer,Vf->Size) == false)
+   auto const backup = DescP;
+   char const * const TagName = "\nDescription";
+   size_t const TagLen = strlen(TagName);
+   while ((DescP = static_cast<char const *>(memchr(DescP, '\n', Length - (DescP - backup)))) != nullptr)
    {
-      delete [] Buffer;
+      if (strncmp(DescP, TagName, TagLen) == 0)
+	 break;
+      else
+	 ++DescP;
+   }
+   if (DescP != nullptr)
+      ++DescP;
+   return DescP;
+}
+									/*}}}*/
+static APT_PURE char const *skipColonSpaces(char const *Buffer, size_t const Length) /*{{{*/
+{
+   // skipping withspace before and after the field-value separating colon
+   char const *const Start = Buffer;
+   for (; isspace(*Buffer) != 0 && Length - (Buffer - Start) > 0; ++Buffer)
+      ;
+   if (*Buffer != ':')
+      return nullptr;
+   ++Buffer;
+   for (; isspace(*Buffer) != 0 && Length - (Buffer - Start) > 0; ++Buffer)
+      ;
+   if (Length - (Buffer - Start) <= 0)
+      return nullptr;
+   return Buffer;
+}
+									/*}}}*/
+
+bool DisplayRecordV1(pkgCacheFile &, pkgRecords &Recs, /*{{{*/
+		     pkgCache::VerIterator const &V, pkgCache::VerFileIterator const &Vf,
+		     char const *Buffer, size_t Length, std::ostream &out)
+{
+   if (unlikely(Length < 4))
       return false;
+
+   auto const Desc = V.TranslatedDescription();
+   if (Desc.end())
+   {
+      /* This handles the unusual case that we have no description whatsoever.
+	 The slightly more common case of only having a short-description embedded
+	 in the record could be handled here, but apt supports also having multiple
+	 descriptions embedded in the record, so we deal with that case later */
+      if (FileFd::Write(STDOUT_FILENO, Buffer, Length) == false)
+	 return false;
+      if (strncmp((Buffer + Length - 4), "\r\n\r\n", 4) != 0 &&
+	  strncmp((Buffer + Length - 2), "\n\n", 2) != 0)
+	 out << std::endl;
+      return true;
    }
 
    // Get a pointer to start of Description field
-   const unsigned char *DescP = (unsigned char*)strstr((char*)Buffer, "\nDescription");
-   if (DescP != NULL)
-      ++DescP;
-   else
-      DescP = Buffer + Vf->Size;
+   char const *DescP = findDescriptionField(Buffer, Length);
+   if (DescP == nullptr)
+      DescP = Buffer + Length;
 
    // Write all but Description
-   size_t const length = DescP - Buffer;
-   if (length != 0 && FileFd::Write(STDOUT_FILENO, Buffer, length) == false)
-   {
-      delete [] Buffer;
+   size_t const untilDesc = DescP - Buffer;
+   if (untilDesc != 0 && FileFd::Write(STDOUT_FILENO, Buffer, untilDesc) == false)
       return false;
-   }
 
    // Show the right description
-   pkgRecords Recs(*Cache);
-   pkgCache::DescIterator Desc = V.TranslatedDescription();
-   if (Desc.end() == false)
-   {
-      pkgRecords::Parser &P = Recs.Lookup(Desc.FileList());
-      out << "Description" << ( (strcmp(Desc.LanguageCode(),"") != 0) ? "-" : "" ) << Desc.LanguageCode() << ": " << P.LongDesc();
-      out << std::endl << "Description-md5: " << Desc.md5() << std::endl;
+   char desctag[50];
+   auto const langcode = Desc.LanguageCode();
+   if (strcmp(langcode, "") == 0)
+      strcpy(desctag, "\nDescription");
+   else
+      snprintf(desctag, sizeof(desctag), "\nDescription-%s", langcode);
 
-      // Find the first field after the description (if there is any)
-      DescP = skipDescriptionFields(DescP);
-   }
-   // else we have no translation, so we found a lonely Description-md5 -> don't skip it
-
-   // write the rest of the buffer, but skip mixed in Descriptions* fields
-   while (DescP != NULL)
+   out << desctag + 1 << ": " << std::flush;
+   auto const Df = Desc.FileList();
+   if (Df.end() == false)
    {
-      const unsigned char * const Start = DescP;
-      const unsigned char *End = (unsigned char*)strstr((char*)DescP, "\nDescription");
-      if (End == NULL)
+      if (Desc.FileList()->File == Vf->File)
       {
-	 End = &Buffer[Vf->Size];
-	 DescP = NULL;
+	 /* If we have the file already open look in the buffer for the
+	    description we want to display. Note that this might not be the
+	    only one we can encounter in this record */
+	 char const *Start = DescP;
+	 do
+	 {
+	    if (strncmp(Start, desctag + 1, strlen(desctag) - 1) != 0)
+	       continue;
+	    Start += strlen(desctag) - 1;
+	    Start = skipColonSpaces(Start, Length - (Start - Buffer));
+	    if (Start == nullptr)
+	       continue;
+	    char const *End = skipDescription(Start, Length - (Start - Buffer), false);
+	    if (likely(End != nullptr))
+	       FileFd::Write(STDOUT_FILENO, Start, End - (Start + 1));
+	    break;
+	 } while ((Start = findDescriptionField(Start, Length - (Start - Buffer))) != nullptr);
       }
       else
       {
-	 ++End; // get the newline into the output
-	 DescP = skipDescriptionFields(End + strlen("Description"));
+	 pkgRecords::Parser &P = Recs.Lookup(Df);
+	 out << P.LongDesc();
       }
+   }
+
+   out << std::endl << "Description-md5: " << Desc.md5() << std::endl;
+
+   // Find the first field after the description (if there is any)
+   DescP = skipDescription(DescP, Length - (DescP - Buffer), true);
+
+   // write the rest of the buffer, but skip mixed in Descriptions* fields
+   while (DescP != nullptr)
+   {
+      char const *const Start = DescP;
+      char const *End = findDescriptionField(DescP, Length - (DescP - Buffer));
+      if (End == nullptr)
+      {
+	 DescP = nullptr;
+	 End = Buffer + Length - 1;
+	 size_t endings = 0;
+	 while (*End == '\n')
+	 {
+	    --End;
+	    if (*End == '\r')
+	       --End;
+	    ++endings;
+	 }
+	 if (endings >= 1)
+	 {
+	    ++End;
+	    if (*End == '\r')
+	       ++End;
+	 }
+	 ++End;
+      }
+      else
+	 DescP = skipDescription(End + strlen("Description"), Length - (End - Buffer), true);
+
       size_t const length = End - Start;
       if (length != 0 && FileFd::Write(STDOUT_FILENO, Start, length) == false)
-      {
-	 delete [] Buffer;
 	 return false;
-      }
    }
    // write a final newline after the last field
    out << std::endl;
 
-   delete [] Buffer;
    return true;
 }
 									/*}}}*/
-static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgCache::VerIterator const &V,/*{{{*/
-                   std::ostream &out)
+static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgRecords &Recs, /*{{{*/
+			    pkgCache::VerIterator const &V, pkgCache::VerFileIterator const &Vf,
+			    char const *Buffer, size_t const Length, std::ostream &out)
 {
-   FileFd PkgF;
-   pkgCache::VerFileIterator Vf;
-   if (OpenPackagesFile(CacheFile, V, PkgF, Vf) == false)
-      return false;
-
    // Check and load the package list file
    pkgCache::PkgFileIterator I = Vf.File();
-   if (I.IsOk() == false)
-      return _error->Error(_("Package file %s is out of sync."),I.FileName());
 
    // find matching sources.list metaindex
    pkgSourceList *SrcList = CacheFile.GetSourceList();
    pkgIndexFile *Index;
    if (SrcList->FindIndex(I, Index) == false &&
        _system->FindIndex(I, Index) == false)
-      return _error->Error("Can not find indexfile for Package %s (%s)", 
-                           V.ParentPkg().Name(), V.VerStr());
+      return _error->Error("Can not find indexfile for Package %s (%s)",
+			   V.ParentPkg().Name(), V.VerStr());
    std::string source_index_file = Index->Describe(true);
 
    // Read the record
    pkgTagSection Tags;
-   pkgTagFile TagF(&PkgF, Vf->Size);
-
-   if (TagF.Jump(Tags, Vf->Offset) == false)
+   if (Tags.Scan(Buffer, Length, true) == false)
       return _error->Error("Internal Error, Unable to parse a package record");
 
    // make size nice
@@ -234,10 +282,6 @@ static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgCache::VerIterator const
       return _error->Error("Internal Error, Unable to parse a package record");
 
    // write the description
-   pkgCache * const Cache = CacheFile.GetPkgCache();
-   if (unlikely(Cache == NULL))
-      return false;
-   pkgRecords Recs(*Cache);
    // FIXME: show (optionally) all available translations(?)
    pkgCache::DescIterator Desc = V.TranslatedDescription();
    if (Desc.end() == false)
@@ -245,7 +289,7 @@ static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgCache::VerIterator const
       pkgRecords::Parser &P = Recs.Lookup(Desc.FileList());
       out << "Description: " << P.LongDesc();
    }
-   
+
    // write a final newline (after the description)
    out << std::endl << std::endl;
 
@@ -255,26 +299,93 @@ static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgCache::VerIterator const
 bool ShowPackage(CommandLine &CmdL)					/*{{{*/
 {
    pkgCacheFile CacheFile;
+   auto VolatileCmdL = GetAllPackagesAsPseudo(CacheFile.GetSourceList(), CmdL, AddVolatileBinaryFile, "");
+
+   if (unlikely(CacheFile.GetPkgCache() == nullptr))
+      return false;
    CacheSetHelperVirtuals helper(true, GlobalError::NOTICE);
    APT::CacheSetHelper::VerSelector const select = _config->FindB("APT::Cache::AllVersions", true) ?
 			APT::CacheSetHelper::ALL : APT::CacheSetHelper::CANDIDATE;
    if (select == APT::CacheSetHelper::CANDIDATE && CacheFile.GetDepCache() == nullptr)
       return false;
-   APT::VersionList const verset = APT::VersionList::FromCommandLine(CacheFile, CmdL.FileList + 1, select, helper);
-   int const ShowVersion = _config->FindI("APT::Cache::Show::Version", 1);
-   for (APT::VersionList::const_iterator Ver = verset.begin(); Ver != verset.end(); ++Ver)
-      if (ShowVersion <= 1)
+
+   APT::VersionList verset;
+   size_t normalPackages = 0;
+   for (auto const &I: VolatileCmdL)
+   {
+      if (I.index == -1)
       {
-	 if (DisplayRecordV1(CacheFile, Ver, std::cout) == false)
-	    return false;
+	 APT::VersionContainerInterface::FromString(&verset, CacheFile, I.name, select, helper);
+	 ++normalPackages;
       }
       else
-	 if (DisplayRecordV2(CacheFile, Ver, c1out) == false)
+      {
+	 if (select != APT::CacheSetHelper::CANDIDATE && unlikely(CacheFile.GetDepCache() == nullptr))
 	    return false;
+	 pkgCache::PkgIterator const P = CacheFile->FindPkg(I.name);
+	 if (unlikely(P.end()))
+	    continue;
 
-   if (select == APT::CacheSetHelper::CANDIDATE)
+	 // Set any version providing the .deb as the candidate.
+	 for (auto Prv = P.ProvidesList(); Prv.end() == false; ++Prv)
+	 {
+	    if (I.release.empty())
+	       CacheFile->SetCandidateVersion(Prv.OwnerVer());
+	    else
+	       CacheFile->SetCandidateRelease(Prv.OwnerVer(), I.release);
+
+	    // via cacheset to have our usual handling
+	    APT::VersionContainerInterface::FromPackage(&verset, CacheFile, Prv.OwnerPkg(), APT::CacheSetHelper::CANDIDATE, helper);
+	 }
+      }
+   }
+
+   int const ShowVersion = _config->FindI("APT::Cache::Show::Version", 1);
+   pkgRecords Recs(CacheFile);
+   for (APT::VersionList::const_iterator Ver = verset.begin(); Ver != verset.end(); ++Ver)
    {
-      APT::VersionList const verset_all = APT::VersionList::FromCommandLine(CacheFile, CmdL.FileList + 1, APT::CacheSetHelper::ALL, helper);
+      pkgCache::VerFileIterator Vf;
+      auto &Parser = LookupParser(Recs, Ver, Vf);
+      char const *Start, *Stop;
+      Parser.GetRec(Start, Stop);
+      size_t const Length = Stop - Start;
+
+      if (ShowVersion <= 1)
+      {
+	 if (DisplayRecordV1(CacheFile, Recs, Ver, Vf, Start, Length, std::cout) == false)
+	    return false;
+      }
+      else if (DisplayRecordV2(CacheFile, Recs, Ver, Vf, Start, Length + 1, c1out) == false)
+	 return false;
+   }
+
+   if (select == APT::CacheSetHelper::CANDIDATE && normalPackages != 0)
+   {
+      APT::VersionList verset_all;
+      for (auto const &I: VolatileCmdL)
+      {
+	 if (I.index == -1)
+	    APT::VersionContainerInterface::FromString(&verset_all, CacheFile, I.name, APT::CacheSetHelper::ALL, helper);
+	 else
+	 {
+	    pkgCache::PkgIterator const P = CacheFile->FindPkg(I.name);
+	    if (unlikely(P.end()))
+	       continue;
+
+	    // Set any version providing the .deb as the candidate.
+	    for (auto Prv = P.ProvidesList(); Prv.end() == false; ++Prv)
+	    {
+	       if (I.release.empty())
+		  CacheFile->SetCandidateVersion(Prv.OwnerVer());
+	       else
+		  CacheFile->SetCandidateRelease(Prv.OwnerVer(), I.release);
+
+	       // via cacheset to have our usual virtual handling
+	       APT::VersionContainerInterface::FromPackage(&verset_all, CacheFile, Prv.OwnerPkg(), APT::CacheSetHelper::CANDIDATE, helper);
+	    }
+	 }
+      }
+
       int const records = verset_all.size() - verset.size();
       if (records > 0)
          _error->Notice(P_("There is %i additional record. Please use the '-a' switch to see it", "There are %i additional records. Please use the '-a' switch to see them.", records), records);
